require 'spec_helper'

describe Speedflow::Plugin::Jira::Plugin do
  let(:plugin) do
    plugin = ::Speedflow::Plugin::Jira::Plugin.new(double, double)
    plugin.prompt = double
    plugin.client = double
    plugin
  end

  it '.action_create_issue' do
    allow(plugin.config)
      .to receive(:by_input)
      .with('title')
      .and_return('Test')
    allow(plugin.config)
      .to receive(:by_input)
      .with('project')
      .and_return('T')
    allow(plugin.config)
      .to receive(:by_input)
      .with('type')
      .and_return('')

    allow(plugin.client)
      .to receive(:issue_types)
      .and_return('3')

    allow(plugin.prompt)
      .to receive(:issue_type)
      .with('3')
      .and_return('3')

    allow(plugin.client)
      .to receive(:issue_link)
      .with('T-1')
      .and_return('xxx')

    return_object = double(key: 'T-1', fields: { 'summary' => 'Test' })
    allow(plugin.client)
      .to receive(:create_issue)
      .with('T', 'Test', '3')
      .and_return(return_object)

    expected_hash = { 'foo' => 'bar' }

    allow(plugin.client)
      .to receive(:issue_output_format)
      .with(return_object)
      .and_return(expected_hash)

    expect(plugin.action_create_issue)
      .to eq expected_hash
  end

  it '.action_search_issue' do
    allow(plugin.config)
      .to receive(:by_input)
      .with('limit', 20)
      .and_return(20)
    allow(plugin.config)
      .to receive(:by_input)
      .with('project')
      .and_return('T')

    allow(plugin.prompt)
      .to receive(:issue)
      .and_yield('test')

    allow(plugin.prompt)
      .to receive(:ok)
      .with('[JIRA] Issue found: T-1')

    allow(plugin.client)
      .to receive(:search_issue)
      .with('T', 'test')
      .and_return(double(take: 'T-1'))

    expected_hash = { 'issue' => { 'key' => 'T-1' } }
    expect(plugin.action_search_issue)
      .to eq expected_hash
  end

  it '.action_change_issue_assignee' do
    allow(plugin.config)
      .to receive(:by_input)
      .with('issue')
      .and_return('T-1')
    allow(plugin.config)
      .to receive(:by_input)
      .with('assignee')
      .and_return('julien.breux')

    allow(plugin.prompt)
      .to receive(:ok)
      .with('[JIRA] Issue assignee changed to: julien.breux')

    allow(plugin.client)
      .to receive(:update_issue_assignee)
      .with('T-1', 'julien.breux')
      .and_return(double(take: 'T-1'))

    expected_hash = {
      'issue' => { 'key' => 'T-1', 'assignee' => 'julien.breux' } }
    expect(plugin.action_change_issue_assignee)
      .to eq expected_hash
  end

  it '.action_change_issue_transition' do
    allow(plugin.config)
      .to receive(:by_input)
      .with('issue')
      .and_return('T-1')
    allow(plugin.config)
      .to receive(:by_input)
      .with('transition')
      .and_return('')

    allow(plugin.client)
      .to receive(:issue_trans)
      .and_return('T-1')

    allow(plugin.prompt)
      .to receive(:transition)
      .with('T-1')
      .and_return('1')

    allow(plugin.client)
      .to receive(:update_issue_transition)
      .with('T-1', '1')

    allow(plugin.prompt)
      .to receive(:ok)
      .with('[JIRA] Issue transition changed to: 1')

    expected_hash = { 'issue' => { 'key' => 'T-1', 'transition' => '1' } }
    expect(plugin.action_change_issue_transition)
      .to eq expected_hash
  end
end
