require 'spec_helper'

describe Speedflow::Plugin::Jira::Prompt do
  let(:prompt) do
    prompt = ::Speedflow::Plugin::Jira::Prompt.new
    prompt.prompt = double
    prompt
  end
  let(:response) do
    double(body: '{"errors": {"k": "v"}, "errorMessages": ["v"]}')
  end
  let(:exception) do
    double(response: response)
  end

  it '.project' do
    allow(prompt.prompt)
      .to receive(:select)
      .with('Choose the project:')
      .and_yield(double(choice: nil))
      .and_return('T')

    projects = [double(name: 'Test', key: 'T')]
    expect(prompt.project(projects)).to eq 'T'
  end

  it '.issue_type' do
    allow(prompt.prompt)
      .to receive(:select)
      .with('Choose the issue type:')
      .and_yield(double(choice: nil))
      .and_return('3')

    issue_types = [double(name: 'Task', id: '3')]
    expect(prompt.issue_type(issue_types)).to eq '3'
  end

  it '.transition' do
    allow(prompt.prompt)
      .to receive(:select)
      .with('Choose the transition:')
      .and_yield(double(choice: nil))
      .and_return('1')

    transitions = [double(name: 'Start', id: '1')]
    expect(prompt.transition(transitions)).to eq '1'
  end

  it '.issue' do
    allow(prompt.prompt)
      .to receive(:select)
      .with('Choose the issue:')
      .and_yield(double(choice: nil))
      .and_return('T-1')
    allow(prompt.prompt)
      .to receive(:ask)
      .with('What is the title of issue?', required: true)
      .and_return('First Task')
    allow(prompt.prompt)
      .to receive(:ok)
      .with('[JIRA] Issue found: T-1')

    issue = prompt.issue do |_|
      [double(summary: 'First Task', key: 'T-1')]
    end
    expect(issue).to eq 'T-1'
  end

  it '.errors' do
    allow(prompt.prompt)
      .to receive(:error).with('Jira errors')
    allow(prompt.prompt)
      .to receive(:warn).with('- k: v')
    allow(prompt.prompt)
      .to receive(:warn).with('- v')

    expect(prompt.errors(exception))
  end
end
